@@ -130,13 +130,9 @@ miner_applications_basedir_add (TrackerMinerFS *fs,
 	g_free (path);
 }
 
-static gboolean
-miner_applications_initable_init (GInitable     *initable,
-                                  GCancellable  *cancellable,
-                                  GError       **error)
+static void
+miner_applications_add_directories (TrackerMinerFS *fs)
 {
-	TrackerMinerFS *fs;
-	GError *inner_error = NULL;
 #ifdef HAVE_MEEGOTOUCH
 	GFile *file;
 	const gchar *path;
@@ -145,14 +141,6 @@ miner_applications_initable_init (GInitable     *initable,
 	const gchar *user_data_dir;
 	gint i;
 
-	fs = TRACKER_MINER_FS (initable);
-
-	/* Chain up parent's initable callback before calling child's one */
-	if (!miner_applications_initable_parent_iface->init (initable, cancellable, &inner_error)) {
-		g_propagate_error (error, inner_error);
-		return FALSE;
-	}
-
 	g_message ("Setting up applications to iterate from XDG system directories");
 
 	/* Add all XDG system and local dirs */
@@ -181,15 +169,82 @@ miner_applications_initable_init (GInitable     *initable,
 	file = g_file_new_for_path (path);
 	tracker_miner_fs_directory_add (fs, file, TRUE);
 	g_object_unref (file);
+#endif /* HAVE_MEEGOTOUCH */
+}
+
+static void
+tracker_locale_notify_cb (TrackerLocaleID id,
+                          gpointer        user_data)
+{
+	TrackerMiner *miner = user_data;
+
+	if (tracker_miner_applications_detect_locale_changed (miner)) {
+		tracker_miner_fs_set_mtime_checking (TRACKER_MINER_FS (miner), TRUE);
+
+		miner_applications_add_directories (TRACKER_MINER_FS (miner));
+	}
+}
+
+static void
+miner_finished_cb (TrackerMinerFS *fs,
+                   gdouble         seconds_elapsed,
+                   guint           total_directories_found,
+                   guint           total_directories_ignored,
+                   guint           total_files_found,
+                   guint           total_files_ignored,
+                   gpointer        user_data)
+{
+	/* Update locale file if necessary */
+	if (tracker_miner_applications_locale_changed ()) {
+		tracker_miner_applications_locale_set_current ();
+	}
+}
+
+static gboolean
+miner_applications_initable_init (GInitable     *initable,
+                                  GCancellable  *cancellable,
+                                  GError       **error)
+{
+	TrackerMinerFS *fs;
+	TrackerMinerApplications *app;
+	GError *inner_error = NULL;
+
+	fs = TRACKER_MINER_FS (initable);
+	app = TRACKER_MINER_APPLICATIONS (initable);
+
+	/* Chain up parent's initable callback before calling child's one */
+	if (!miner_applications_initable_parent_iface->init (initable, cancellable, &inner_error)) {
+		g_propagate_error (error, inner_error);
+		return FALSE;
+	}
+
+	g_signal_connect (fs, "finished",
+	                  G_CALLBACK (miner_finished_cb),
+	                  NULL);
+
+	miner_applications_add_directories (fs);
+
+#ifdef HAVE_MEEGOTOUCH
 	tracker_miner_applications_meego_init ();
 #endif /* HAVE_MEEGOTOUCH */
 
+	app->locale_notification_id = tracker_locale_notify_add (TRACKER_LOCALE_LANGUAGE,
+	                                                         tracker_locale_notify_cb,
+	                                                         app,
+	                                                         NULL);
+
 	return TRUE;
 }
 
 static void
 miner_applications_finalize (GObject *object)
 {
+	TrackerMinerApplications *app;
+
+	app = TRACKER_MINER_APPLICATIONS (object);
+
+	tracker_locale_notify_remove (app->locale_notification_id);
+
 #ifdef HAVE_MEEGOTOUCH
 	tracker_miner_applications_meego_shutdown ();
 #endif /* HAVE_MEEGOTOUCH */
@@ -437,12 +492,15 @@ process_desktop_file (ProcessApplicationData  *data,
 		tracker_sparql_builder_object (sparql, "nfo:SoftwareCategory");
 
 		if (icon) {
+			gchar *escaped_icon;
 			gchar *icon_uri;
 
 			/* Sanitize icon */
 			g_strstrip (icon);
 
-			icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", icon);
+			escaped_icon = g_uri_escape_string (icon, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
+
+			icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", escaped_icon);
 
 			tracker_sparql_builder_subject_iri (sparql, icon_uri);
 			tracker_sparql_builder_predicate (sparql, "a");
@@ -453,6 +511,7 @@ process_desktop_file (ProcessApplicationData  *data,
 			tracker_sparql_builder_object_iri (sparql, icon_uri);
 
 			g_free (icon_uri);
+			g_free (escaped_icon);
 			g_free (icon);
 		}
 
@@ -473,6 +532,30 @@ process_desktop_file (ProcessApplicationData  *data,
 
 		tracker_sparql_builder_predicate (sparql, "nie:dataSource");
 		tracker_sparql_builder_object_iri (sparql, APPLICATION_DATASOURCE_URN);
+	} else if (name && g_ascii_strcasecmp (type, "Link") == 0) {
+		gchar *url = g_key_file_get_string (key_file, GROUP_DESKTOP_ENTRY, "URL", NULL);
+
+		if (url) {
+			uri = g_file_get_uri (data->file);
+			tracker_sparql_builder_insert_silent_open (sparql, TRACKER_MINER_FS_GRAPH_URN);
+
+			tracker_sparql_builder_subject_iri (sparql, uri);
+			tracker_sparql_builder_predicate (sparql, "a");
+			tracker_sparql_builder_object (sparql, "nfo:Bookmark");
+
+			tracker_sparql_builder_predicate (sparql, "nfo:bookmarks");
+			tracker_sparql_builder_object_iri (sparql, url);
+
+			tracker_sparql_builder_predicate (sparql, "nie:dataSource");
+			tracker_sparql_builder_object_iri (sparql, APPLICATION_DATASOURCE_URN);
+
+			is_software = FALSE;
+
+			g_free (url);
+		} else {
+			g_warning ("Invalid desktop file: '%s'", uri);
+			g_warning ("  Type 'Link' requires a URL");
+		}
 #ifdef HAVE_MEEGOTOUCH
 	} else if (name && g_ascii_strcasecmp (type, "ControlPanelApplet") == 0) {
 		/* Special case control panel applets */
@@ -610,12 +693,15 @@ process_desktop_file (ProcessApplicationData  *data,
 			icon = g_key_file_get_string (key_file, GROUP_DESKTOP_ENTRY, "Icon", NULL);
 
 			if (icon) {
+				gchar *escaped_icon;
 				gchar *icon_uri;
 
 				/* Sanitize icon */
 				g_strstrip (icon);
 
-				icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", icon);
+				escaped_icon = g_uri_escape_string (icon, G_URI_RESERVED_CHARS_ALLOWED_IN_PATH, FALSE);
+
+				icon_uri = g_strdup_printf (THEME_ICON_URN_PREFIX "%s", escaped_icon);
 
 				tracker_sparql_builder_subject_iri (sparql, icon_uri);
 				tracker_sparql_builder_predicate (sparql, "a");
@@ -626,6 +712,7 @@ process_desktop_file (ProcessApplicationData  *data,
 				tracker_sparql_builder_object_iri (sparql, icon_uri);
 
 				g_free (icon_uri);
+				g_free (escaped_icon);
 				g_free (icon);
 			}
 		}
@@ -751,10 +838,22 @@ process_file_cb (GObject      *object,
 
 	if (g_file_info_get_file_type (file_info) == G_FILE_TYPE_DIRECTORY) {
 		process_directory (data, file_info, &error);
-	} else if (data->key_file) {
-		process_desktop_file (data, file_info, &error);
 	} else {
-		error = g_error_new_literal (miner_applications_error_quark, 0, "File is not a key file");
+		data->key_file = get_desktop_key_file (file, &data->type, &error);
+		if (!data->key_file) {
+			gchar *uri;
+
+			uri = g_file_get_uri (file);
+			g_warning ("Couldn't properly parse desktop file '%s': '%s'",
+			           uri,
+			           error ? error->message : "unknown error");
+			g_free (uri);
+			g_clear_error (&error);
+
+			error = g_error_new_literal (miner_applications_error_quark, 0, "File is not a key file");
+		} else {
+			process_desktop_file (data, file_info, &error);
+		}
 	}
 
 	tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
@@ -776,19 +875,13 @@ miner_applications_process_file (TrackerMinerFS       *fs,
                                  GCancellable         *cancellable)
 {
 	ProcessApplicationData *data;
-	GKeyFile *key_file;
 	const gchar *attrs;
-	gchar *type;
-
-	key_file = get_desktop_key_file (file, &type, NULL);
 
 	data = g_slice_new0 (ProcessApplicationData);
 	data->miner = g_object_ref (fs);
 	data->sparql = g_object_ref (sparql);
 	data->file = g_object_ref (file);
 	data->cancellable = g_object_ref (cancellable);
-	data->key_file = key_file;
-	data->type = type;
 
 	attrs = G_FILE_ATTRIBUTE_TIME_MODIFIED ","
 		G_FILE_ATTRIBUTE_STANDARD_TYPE;
